'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var d3 = require('d3');
var _ = require('lodash');

var D3Tree = function () {
  function D3Tree(options) {
    _classCallCheck(this, D3Tree);

    if (options.data.data && options.data.data.image) {
      options.data.image = options.data.data.image;
    }
    if (options.data.data && options.data.data.text) {
      options.data.text = options.data.data.text;
    }

    this._transtromTree(options.data.children);
    this.tree = null;
    this.root = null;
    this.container = null;
    this.layerDepth = null;
    this.maxLayerDepth = 0;
    this.resourceHeight = 100;
    this.resourceWidth = 100;
    this.transition = null;
    this.textLength = 0;
    this.fontSize = 14;
    this.lineHeight = 1.5;

    this.options = Object.assign({
      width: 800,
      height: 600,
      duration: 0,
      selector: 'body',
      prefixClass: 'd3-tree',
      data: {},
      resourceHeight: 200,
      resourceWidth: 200,
      imageMargin: 10
    }, options);
  }

  _createClass(D3Tree, [{
    key: '_transtromTree',
    value: function _transtromTree(suites) {
      var _this = this;

      suites && suites.forEach(function (suite, index) {
        if (suite.data && suite.data.image) {
          suite.image = suite.data.image;
        }
        if (suite.data && suite.data.text) {
          suite.text = suite.data.text;
        }

        if (suite.children) {
          _this._transtromTree(suite.children);
        }
      });
      return suites;
    }
  }, {
    key: 'init',
    value: function init() {
      this.container = d3.select(this.options.selector).append('svg').attr('width', this.options.width).attr('height', this.options.height).append('g');
      this.renderTree();
    }
  }, {
    key: 'multiline',
    value: function multiline() {
      return function (selection) {
        selection.each(function (d, i) {
          var text = d3.select(this);
          var lines = (d.data.text || '').split(/\n/);
          var lineCount = lines.length;
          var isMulti = lineCount !== 1;
          for (var _i = 0; _i < lineCount; _i++) {
            var line = lines[_i];
            text.append('tspan').attr('x', 0).attr('dy', isMulti ? '1.5em' : '0').text(line);
          }
        });
      };
    }
  }, {
    key: 'findMaxLayer',
    value: function findMaxLayer(root, layer, maxLayerDepth) {
      var _this2 = this;

      if (root.children) {
        root.children.forEach(function (child) {
          var childTextLength = child.data.text ? child.data.text.length : 0;
          var lines = (child.data.text || '').split(/\n/);
          var lineCount = lines.length;
          var isMulti = lineCount !== 1;
          if (isMulti) {
            var line = _.maxBy(lines, function (line) {
              return escape(line).length;
            });
            childTextLength = line.length;
          }
          _this2.textLength = _this2.textLength < childTextLength ? childTextLength : _this2.textLength;
          maxLayerDepth[layer + 1]++;
          _this2.findMaxLayer(child, layer + 1, maxLayerDepth);
        });
      }
      return maxLayerDepth;
    }
  }, {
    key: 'checkResourceHeight',
    value: function checkResourceHeight(root) {
      var _this3 = this;

      if (root.children) {
        var imageMargin = root.children.length > 1 ? root.children[1].x - root.children[0].x : 0;
        if (this.resourceHeight + this.options.imageMargin > imageMargin && imageMargin) {
          this.resourceHeight = imageMargin - this.options.imageMargin;
          this.resourceWidth = this.resourceHeight * this.options.resourceWidth / this.options.resourceHeight;
        }
        root.children.forEach(function (child) {
          _this3.checkResourceHeight(child);
        });
      }
    }
  }, {
    key: 'renderTree',
    value: function renderTree() {
      var _this4 = this;

      var that = this;
      this.root = d3.hierarchy(this.options.data);
      this.layerDepth = Array(this.root.height + 1).fill(0);
      this.maxLayerDepth = this.findMaxLayer(this.root, 0, this.layerDepth);
      this.resourceHeight = this.options.height / Math.max.apply(Math, _toConsumableArray(this.maxLayerDepth)) >= this.options.resourceHeight ? this.options.resourceHeight : this.options.height / Math.max.apply(Math, _toConsumableArray(this.maxLayerDepth)) - 10;
      this.resourceWidth = this.resourceHeight * this.options.resourceWidth / this.options.resourceHeight;
      this.tree = d3.tree().size([this.options.height, this.options.width - this.resourceWidth - this.fontSize * this.textLength]);
      this.tree(this.root);
      this.checkResourceHeight(this.root);

      this.container.selectAll('.' + this.options.prefixClass + '-link').data(this.root.descendants().slice(1)).enter().append('path').attr('class', this.options.prefixClass + '-link').attr('d', function (d) {
        return 'M' + d.y + ',' + d.x + 'C' + (d.parent.y + _this4.resourceWidth) + ',' + d.x + ' ' + (d.parent.y + _this4.resourceWidth + 110) + ',' + d.parent.x + ' ' + (d.parent.y + _this4.resourceWidth + 10) + ',' + d.parent.x;
      });

      var node = this.container.selectAll('.' + this.options.prefixClass + '-node').data(this.root.descendants()).enter().append('g').attr('class', this.options.prefixClass + '-node').attr('transform', function (d) {
        return 'translate(' + d.y + ',' + d.x + ')';
      }).each(function (d) {
        var _this5 = this;

        var resourceList = d.data.image;
        if (resourceList && !!~resourceList.indexOf('[')) {
          resourceList = resourceList.replace(/(\[\n|\n\]| )/g, '').split('\n');
        } else {
          resourceList = [resourceList];
        }

        var resourceWidth = that.resourceWidth / resourceList.length;
        var resourceHeight = that.resourceHeight / resourceList.length;

        resourceList.forEach(function (resource, index) {
          if (resource && resource.endsWith('.webm')) {
            d3.select(_this5).append('video').attr('src', resource).attr('type', 'video/webm').attr('controls', 'controls').attr('data-index', index).attr('style', function (d) {
              return 'width: ' + resourceWidth + 'px; height: ' + resourceHeight + 'px';
            }).attr('transform', function (d) {
              return 'translate(' + resourceWidth * index + ', -' + resourceHeight / 2 + ')';
            });
          } else {
            d3.select(_this5).append('image').attr('xlink:href', resource).attr('data-index', index).attr('height', function (d) {
              return resourceHeight;
            }).attr('transform', function (d) {
              return 'translate(' + resourceWidth * index + ', -' + resourceHeight / 2 + ')';
            }).attr('width', function (d) {
              return resourceWidth;
            });
          }
        });
      });

      node.append('text').attr('fill', '#111').call(this.multiline()).attr('transform', function (d) {
        var lines = (d.data.text || '').split(/\n/);
        var lineCount = lines.length;
        var isMulti = lineCount !== 1;
        var heightHalf = isMulti ? _this4.fontSize * _this4.lineHeight * (lineCount + 1) / 2 : 0;
        if (d.data.image) {
          return 'translate(' + (that.resourceWidth + 10) + ', ' + -heightHalf + ')';
        } else {
          var textLength = d.data && d.data.text && d.data.text.length || 0;
          return 'translate(' + (that.resourceWidth / 2 - 2 * textLength) + ', ' + -heightHalf + ')';
        }
      });
    }
  }, {
    key: 'update',
    value: function update(data) {
      var _this6 = this;

      this.options.data = data;
      this.renderTree();
      this.transition = this.container.transition().duration(this.options.duration);
      this.transition.selectAll('.' + this.options.prefixClass + '-link').attr('d', function (d) {
        return 'M' + d.y + ',' + d.x + 'C' + (d.parent.y + _this6.resourceWidth) + ',' + d.x + ' ' + (d.parent.y + _this6.resourceWidth + 110) + ',' + d.parent.x + ' ' + (d.parent.y + _this6.resourceWidth + 10) + ',' + d.parent.x;
      });
      this.transition.selectAll('.' + this.options.prefixClass + '-node').attr('transform', function (d) {
        return 'translate(' + d.y + ',' + d.x + ')';
      });
      this.transition.selectAll('text').attr('transform', function (d) {
        if (d.data.image) {
          return 'translate(' + (_this6.resourceWidth + 10) + ', 4)';
        } else {
          var textLength = d.data && d.data.text && d.data.text.length || 0;
          return 'translate(' + (_this6.resourceWidth / 2 - 2 * textLength) + ', 4)';
        }
      });
    }
  }]);

  return D3Tree;
}();

module.exports = D3Tree;